/*
 * File:		ns8390.h
 * Purpose:		Header file for the National Semiconductor NS8390
 *			Ethernet chipset, commonly used in NE2000-type cards.
 *
 * Notes:
 *
 * Author:		Eric DeVolder
 * Date:		12-15-95
 *
 * Modifications:
 *                      Shamelessly stolen for RTEMS and slimed down to just the struct and defs - DJF
 *
 */

#ifndef _NS8390_H
#define _NS8390_H

/*
 * The MCF5206 EVS board has a screwed up ISA bus interface.
 * The ISA bus on this board only handles 8 bits, 16 bits WILL
 * NOT WORK!!!  In addition, the odd and even addresses on this
 * bus are not sequential.  Instead, even addresses appear at
 * beginning at 0x0000, and odd addresses appear beginning at
 * 0x8000.
 */

typedef volatile struct
{
	rtems_unsigned16	_CR; /* Had to insert a "_" as CR appears to be a predefined macro? - DJF */
	union
	{
		struct
		{
			/* Even registers */
			rtems_unsigned16	CLDA1;		/* CLDA1 (rd) PSTOP (wr) */
			rtems_unsigned16	TSR;		/* TSR (rd) TPSR (wr) */
			rtems_unsigned16	FIFO;		/* FIFO (rd) TBCR1 (wr) */
			rtems_unsigned16	CRDA0;		/* CRDA0 (rd) RSAR0 (wr) */
			rtems_unsigned16	RBCR0;		/* Remote Byte Count 0 (wr) */
			rtems_unsigned16	RSR;		/* RSR (rd) RCR (wr) */
			rtems_unsigned16	CNTR1;		/* CNTR1 (rd) DCR (wr) */

			rtems_unsigned16	DATAPORT;

			rtems_unsigned16	reserved[(0x10000-0x0012)/2];

			/* Odd registers */
			rtems_unsigned16	CLDA0;		/* CLDA0 (rd) PSTART (wr) */
			rtems_unsigned16	BNRY;		/* Boundary pointer (rd wr) */
			rtems_unsigned16	NCR;		/* NCR (rd) TBCR0 (wr) */
			rtems_unsigned16	ISR;		/* Interrupt Status Register (rd wr) */
			rtems_unsigned16	CRDA1;		/* CRDA1 (rd) RSAR1 (wr) */
			rtems_unsigned16	RBCR1;		/* Remote Byte Count 1 (wr) */
			rtems_unsigned16	CNTR0;		/* CNTR0 (rd) TCR (wr) */
			rtems_unsigned16	CNTR2;		/* CNTR2 (rd) IMR (wr) */
		} page0;
		struct
		{
			/* Even registers */
			rtems_unsigned16	PAR1;		/* Physical Address Byte 1 */
			rtems_unsigned16	PAR3;		/* Physical Address Byte 3 */
			rtems_unsigned16	PAR5;		/* Physical Address Byte 5 */
			rtems_unsigned16	MAR0;		/* Multicast Address Byte 0 */
			rtems_unsigned16	MAR2;		/* Multicast Address Byte 2 */
			rtems_unsigned16	MAR4;		/* Multicast Address Byte 4 */
			rtems_unsigned16	MAR6;		/* Multicast Address Byte 6 */

			rtems_unsigned16	reserved[(0x10000-0x0010)/2];

			/* Odd registers */
			rtems_unsigned16	PAR0;		/* Physical Address Byte 0 */
			rtems_unsigned16	PAR2;		/* Physical Address Byte 2 */
			rtems_unsigned16	PAR4;		/* Physical Address Byte 4 */
			rtems_unsigned16	CURR;		/* Current Page Register (rd wr) */
			rtems_unsigned16	MAR1;		/* Multicast Address Byte 1 */
			rtems_unsigned16	MAR3;		/* Multicast Address Byte 3 */
			rtems_unsigned16	MAR5;		/* Multicast Address Byte 5 */
			rtems_unsigned16	MAR7;		/* Multicast Address Byte 7 */
		} page1;
		struct
		{
			/* Even registers */
			rtems_unsigned16	PSTOP;		/* PSTOP (rd) CLDA1 (wr) */
			rtems_unsigned16	TPSR;		/* Transmit Page Start Address (rd) */
			rtems_unsigned16	ACU;		/* Address Counter Upper */
			rtems_unsigned16	reserved0;
			rtems_unsigned16	reserved2;
			rtems_unsigned16	RCR;		/* Receive Configuration Register (rd) */
			rtems_unsigned16	DCR;		/* Data Configuration Register (rd) */

			rtems_unsigned16	reserved[(0x10000-0x0010)/2];

			/* Odd registers */
			rtems_unsigned16	PSTART;		/* PSTART (rd) CLDA0 (wr) */
			rtems_unsigned16	RNPP;		/* Remote Next Packet Pointer */
			rtems_unsigned16	LNPP;		/* Local Next Packet Pointer */
			rtems_unsigned16	ACL;		/* Address Counter Lower */
			rtems_unsigned16	reserved1;
			rtems_unsigned16	reserved3;
			rtems_unsigned16	TCR;		/* Transmit Configuration Register (rd) */
			rtems_unsigned16	IMR;		/* Interrupt Mask Register (rd) */
		} page2;
	} regs;
} NS8390;

#define NS8390_DATAPORT(PTR)	(PTR->regs.page0.DATAPORT)


/*
 * Bit pattern definitions for the chipset.
 */

/* Command Register, CR (rd wr) */
#define NS8390_CR_STP	(0x01)
#define NS8390_CR_STA	(0x02)
#define NS8390_CR_TXP	(0x04)
#define NS8390_CR_RD_RD	(0x08)
#define NS8390_CR_RD_WR	(0x10)
#define NS8390_CR_RD_SP	(0x11)
#define NS8390_CR_RD_AB	(0x20)
#define NS8390_CR_PAGE0	(0x00)
#define NS8390_CR_PAGE1	(0x40)
#define NS8390_CR_PAGE2	(0x80)
#define NS8390_CR_PAGE3	(0xC0)

/* Transmit Control Register, TCR (wr) */
#define NS8390_TCR_CRC		(0x01)
#define NS8390_TCR_LOOP_NO	(0x00)
#define NS8390_TCR_LOOP_I0	(0x02)
#define NS8390_TCR_LOOP_E1	(0x04)
#define NS8390_TCR_LOOP_E0	(0x06)
#define NS8390_TCR_ATD		(0x08)
#define NS8390_TCR_OFST		(0x10)

/* Data Configuration Register, DCR (wr) */
#define NS8390_DCR_WTS		(0x01)
#define NS8390_DCR_BOS		(0x02)
#define NS8390_DCR_LAS		(0x04)
#define NS8390_DCR_LS		(0x08)
#define NS8390_DCR_AR		(0x10)
#define NS8390_DCR_FT_2B	(0x00)
#define NS8390_DCR_FT_4B	(0x20)
#define NS8390_DCR_FT_8B	(0x40)
#define NS8390_DCR_FT_12B	(0x60)
#define NS8390_DCR_FT_1W	(0x00)
#define NS8390_DCR_FT_2W	(0x20)
#define NS8390_DCR_FT_4W	(0x40)
#define NS8390_DCR_FT_6W	(0x60)

/* Receive Configuration Register, RCR (wr) */
#define NS8390_RCR_SEP		(0x01)
#define NS8390_RCR_AR		(0x02)
#define NS8390_RCR_AB		(0x04)
#define NS8390_RCR_AM		(0x08)
#define NS8390_RCR_PRO		(0x10)
#define NS8390_RCR_MON		(0x20)

/* Interrupt Status Register, ISR (rd wr) */
#define NS8390_ISR_PRX		(0x01)
#define NS8390_ISR_PTX		(0x02)
#define NS8390_ISR_RXE		(0x04)
#define NS8390_ISR_TXE		(0x08)
#define NS8390_ISR_OVW		(0x10)
#define NS8390_ISR_CNT		(0x20)
#define NS8390_ISR_RDC		(0x40)
#define NS8390_ISR_RST		(0x80)

/* Interrupt Mask Register, IMR (wr) */
#define NS8390_IMR_PRXE		(0x01)
#define NS8390_IMR_PTXE		(0x02)
#define NS8390_IMR_RXEE		(0x04)
#define NS8390_IMR_TXEE		(0x08)
#define NS8390_IMR_OVWE		(0x10)
#define NS8390_IMR_CNTE		(0x20)
#define NS8390_IMR_RDCE		(0x40)

/* Received Status Register, RSR, and Packet Status Byte */
#define NS8390_RSR_PRX		(0x01)
#define NS8390_RSR_CRC		(0x02)
#define NS8390_RSR_FAE		(0x04)
#define NS8390_RSR_FO		(0x08)
#define NS8390_RSR_MPA		(0x10)
#define NS8390_RSR_PHY		(0x20)
#define NS8390_RSR_DIS		(0x40)
#define NS8390_RSR_DFR		(0x80)

/* Transmit Status Register, TSR (rd) */
#define NS8390_TSR_PTX		(0x01)
#define NS8390_TSR_COL		(0x04)
#define NS8390_TSR_ABT		(0x08)
#define NS8390_TSR_CRS		(0x10)
#define NS8390_TSR_FU		(0x20)
#define NS8390_TSR_CDH		(0x40)
#define NS8390_TSR_OWC		(0x80)

/*******************************************************************/

#endif /* _NS8390_H */

